function autosplit
%
%***************************************************************************
%                              AutoSplit
%   This source code is (c) Copyright 2002 by Jia-Yu Pan (Computer
%   Science Department, Carnegie Mellon University).
%   It may not be redistributed without the express consent of the 
%   author.
%***************************************************************************

	 
%path(path, '/usr0/jypan/ICA_Demo/');

%%%%%%%%%%%%%%%%%
%%% Load a data set 
%%%%%%%%%%%%%%%%%
%% Synthetic x-shape

%thisFile = 'synthX.mat';
%load(thisFile);  % this works under linux, but not windows

thisFile = 'synthX.txt';  % use ASCII format, for better compatibility
X = importdata(thisFile);
X = X';
% this gives a 2-by-n 'X', the 2-D synthetic X-shape.

outDir = 'result/';
Amat = [outDir, 'synthX_A.mat'];
blocksize = 300;
loglikelihoodEPS = [outDir, 'synthX_likelihood.eps'];
iter_timeEPS = [outDir, 'synthX_timeVSiteration.eps'];

finalResult_EPS = [outDir, 'synthX_A_final.eps'];

[Xr, Xc] = size(X);
[Xr, Xc]


%%% preprocessing
meanX = mean(X,2);
stdX = std(X,0,2);

%XX = (X - repmat(meanX, 1, Xc)); % for plotting
%X = (X - repmat(meanX, 1, Xc)) ./ repmat(stdX, 1, Xc); % ica computation

%meanX, pause


%%% Initialization
A = eye(Xr);
stepsize = 0.003;  % for the first 1000 iterations

%blocksize = 1000;
tolerance = 0.0000001;

allAs =[];

%%% iteration
found = 0;
nIter = 0;
nPlot = 0;
time_per_iter = [];
while(found == 0)
  
  nIter = nIter + 1;
  
  allAs(:,:,nIter) = A;
  
  rind = unidrnd(blocksize, blocksize, 1);
  samX = X(:, rind);
  
  %%% X = AS, => S = A\X 
  %A_inv = inv(A);
  %S = A_inv * samX;

  %%% process of each iteration starts here %%%
  tic;
  
  S = A\samX;
  
  Z = -sign(S);
  deltaA = - A * Z * S' - blocksize * A;
  update = stepsize * deltaA;
  
  %%% update
  if(abs(update) <= tolerance)
    found = 1;
  else
    A = A + update;
    
    if(nIter == 1000)
      stepsize = 0.001;
    end

    decay_period = 500;
    %decay_period = 1000;
    
    if(mod(nIter, decay_period) == 0)
      stepsize = stepsize / sqrt(2);
    end

    %%% Plotting control
    L = loglike(A, S);
    if(mod(nIter, 100) == 0)
      nPlot = nPlot + 1;
      LL(nPlot) = L;
      
      %fprintf('%d %f.\n', nIter, L);
    end
  end
  
  thisTime = toc;
  
  time_per_iter = [time_per_iter; thisTime];
  
end

%%% found A!
save(Amat, 'A');

%%% display mixing matrix A
A

%%%%%%%%%%%%%%%%
%%% plotting %%%
%%%%%%%%%%%%%%%%

%% plot the time per iteration
figure(6);
clf;
plot(1:nIter, time_per_iter);
print('-depsc', iter_timeEPS);


%% plot
figure(1);
clf;
plot(1:nPlot, LL, '+-');
print('-f1', '-depsc', loglikelihoodEPS);


figure(2);
clf;
plotA(A);   
%print('-f2', '-depsc', 'basisvectors.eps');


figure(3);
clf;
thisA = allAs(:,:,nIter);
%scale = 2;
plot(X(1,:), X(2,:), '.', 'MarkerEdgeColor', [0.4, 0.4, 0.4], 'MarkerSize', 3);
hold on;
plot([0 thisA(1,1)], [0, thisA(2,1)], 'k-', 'LineWidth', 6);
plot([0 thisA(1,2)], [0, thisA(2,2)], 'k-', 'LineWidth', 6);
hold off;

print('-depsc', finalResult_EPS);

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% Stop here %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% allAs %%%
size(allAs)
nIter

figure(3);
for i=[1:50:2001, nIter]
  
  thisA = allAs(:,:,i);
  %scale = 2;
  clf;
  plot(X(1,:), X(2,:), '.', 'MarkerEdgeColor', [0.4, 0.4, 0.4], 'MarkerSize', 3);
  hold on;
  plot([0 thisA(1,1)], [0, thisA(2,1)], 'k-', 'LineWidth', 6);
  plot([0 thisA(1,2)], [0, thisA(2,2)], 'k-', 'LineWidth', 6);
  hold off;

  pause(1);
  
  %i, pause
  
  allA_epsFile = sprintf('%sXshape_allA%05d.eps', outDir, i);
  allA_jpgFile = sprintf('%sXshape_allA%05d.jpg', outDir, i);

  %print('-depsc', allA_epsFile);
  %print('-djpeg', allA_jpgFile);
end


return;

